/*
 * Copyright (c) 2002 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: pci.c,v 1.7 2002/01/11 10:32:47 pva Exp $
 * PCI autoconfiguration code
 */

#include <err.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#ifndef LIBPCI
static int pcifd = -1;
#endif /* LIBPCI */

#if defined __OpenBSD__ || defined __NetBSD__ || defined __FreeBSD__
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/pciio.h>

static struct pci_io pi;

void
closepcifd(void) {
	if (close(pcifd) < 0)
		warn("/dev/pci close error");
}
#elif defined linux
#ifdef LIBPCI
#include <pci.h>
#else
#include <stdio.h>
#include <string.h>

u_int32_t
parse(char *source) {
	char str[12];

	strcpy(str, "0x");
	strcat(str, source);
	str[11] = '\0';

	return strtoul(str, (char **)NULL, 16);
}
#endif /* LIBPCI */
#endif

u_int32_t
pci_bus_locate(u_int16_t v /* vendor id */, u_int16_t p /* product id */) {
#if defined __OpenBSD__ || defined __NetBSD__ || defined __FreeBSD__
	u_int32_t device;
	u_int8_t b /* bus */, d /* device */, f /* function */;

	device = (p << 16) | v;

	if ((pcifd = open("/dev/pci", O_RDONLY)) < 0) {
		warn("/dev/pci open error");
		return 0ul;
	}

	/* Prepare the pci_io structure */
	pi.pi_width = 4;
	pi.pi_reg = 0;

	for (b = 0; b < 2; b++) {
		for (f = 0; f < 8; f++) {
			for (d = 0; d < 32; d++) {
				pi.pi_sel.pc_bus = b;
				pi.pi_sel.pc_dev = d;
				pi.pi_sel.pc_func = f;
				pi.pi_reg = 0;
				if (ioctl(pcifd, PCIOCREAD, &pi) < 0) {
					warn("PCIOCREAD");
				} else {
					if (pi.pi_data == device) {
						pi.pi_reg = 16;
						if (ioctl(pcifd, PCIOCREAD, &pi) < 0) {
							warn("PCIOCREAD");
							closepcifd();
							return 0ul;
						}
						closepcifd();
						return (pi.pi_data - 1);
					}
				}
			}
		}
	}
#elif defined linux
#ifdef LIBPCI
	struct pci_access *pa;
	struct pci_dev *dev;
	u_int32_t ret = 0ul;

	pa = pci_alloc();
	pa->method = PCI_ACCESS_I386_TYPE1;

	pci_init(pa);

	pci_scan_bus(pa);

	dev = pa->devices;
	while (dev) {
		if (dev->vendor_id == v && dev->device_id == p) {
			ret = dev->base_addr[0];
			break;
		}
		dev = dev->next;
	}

	pci_cleanup(pa);
	return ret;
#else
	char buf[25];
	char a[5], b[9], c[2], d[9];
	char ch[1];

	if ((pcifd = open("/proc/bus/pci/devices", O_RDONLY)) < 0) {
		warn("/proc/bus/pci/devices open error");
		return 0ul;
	}

	while (1) {
		if (read(pcifd, buf, 24) < 24)
			break;
		buf[24] = '\0';

		/* Even if sscanf fails try to continue */
		sscanf(buf, "%s\t%s\t%s\t%s", a, b, c, d);

		if (parse(b) == ((v << 16) | p)) {
			if (close(pcifd) < 0)
				warn("/proc/bus/pci/devices close error");
			return parse(d) - 1;
		}
		do {
			if (read(pcifd, ch, 1) < 0) {
				warn("/proc/bus/pci/devices read error");
				return 0ul;
			}
		} while (*ch != '\n');
	}

	if (close(pcifd) < 0)
		warn("/proc/bus/pci/devices close error");
#endif /* LIBPCI */
#else
	warnx("this OS is not supported yet");
#endif

	return 0ul;
}
