/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: specs.h,v 1.18 2002/01/14 10:21:40 pva Exp $
 *
 * specs.h -- database of supported cards
 */

#ifndef SPECS_H__
#define SPECS_H__

#include "io.h"

unsigned aztech_ports[] = {0x350, 0x358};
unsigned radiotrack_ports[] = {0x20c, 0x30c};
unsigned bktr_ports[] = {ERADIO_NONE};
unsigned gemtek_isa_ports[] = {0x20c, 0x30c, 0x24c, 0x34c, 0x248};
unsigned pci_ports[] = {ERADIO_NONE};
unsigned sf16fm_ports[] = {0x284, 0x384};
unsigned sv16gold_ports[] = {0x28c};
unsigned bmc_ports[] = {0x20f, 0x30f};

const radiocard_t card[] = {
	{ "az", aztech_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"Aztech/PackardBell FM Radio Card", 8750, 10800,
	Grab_Port_aztech, Release_Port_aztech, info_port_aztech,
	get_freq_stub, get_vol_stub,
	Change_Frequency_aztech,
	mute_aztech, volumeup_aztech, volumedn_aztech,
	state_aztech,
	mono_aztech,
	search_up_generic, search_down_generic},

	{ "zx", radiotrack_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"Zoltrix Radio Plus FM Radio Card", 8800, 10800,
	Grab_Port_zoltrix, Release_Port_zoltrix, info_port_zoltrix,
	get_freq_stub, get_vol_stub,
	Change_Frequency_zoltrix,
	mute_zoltrix, volumeup_zoltrix, volumedn_zoltrix,
	state_zoltrix,
	mono_zoltrix,
	search_up_generic, search_down_generic},

	{ "rt", radiotrack_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"AIMS Lab RadioTrack/RadioReveal FM Radio Card", 8750, 10800,
	Grab_Port_radiotrack, Release_Port_radiotrack, info_port_radiotrack,
	get_freq_stub, get_vol_stub,
	Change_Frequency_radiotrack,
	mute_radiotrack, volumeup_radiotrack, volumedn_radiotrack,
	state_radiotrack,
	mono_radiotrack,
	search_up_generic, search_down_generic},

	{ 
#ifdef linux
	"v4l",
#else
	"bktr",
#endif /* linux */
	bktr_ports, INFO_VOLU | INFO_FREQ | 0x01,
#ifdef linux
	"Video4Linux Driver",
#else
	"Brooktree Bt848 Driver",
#endif /* linux */
	8750, 10800,
	Grab_Port_bktr, Release_Port_bktr, info_port_bktr,
	get_freq_bktr, get_vol_bktr,
	Change_Frequency_bktr,
	mute_bktr, volumeup_bktr, volumedn_bktr,
	state_bktr,
	mono_bktr,
	search_up_generic, search_down_generic},

	{ "gti", gemtek_isa_ports, INFO_ROOT | INFO_SCAN | 0x05,
	"GemTek ISA FM Radio Card", 8700, 10800,
	Grab_Port_gemtek, Release_Port_gemtek, info_port_gemtek,
	get_freq_stub, get_vol_stub,
	Change_Frequency_gemtek,
	mute_gemtek, volumeup_gemtek, mute_gemtek,
	state_gemtek,
	mono_stub,
	search_up_generic, search_down_generic},

	{ "svg", sv16gold_ports, INFO_ROOT | INFO_SCAN | 0x01,
	"Sound Vision 16 Gold ISA Driver", 8700, 10800,
	Grab_Port_gemtek, Release_Port_gemtek, info_port_gemtek,
	get_freq_stub, get_vol_stub,
	Change_Frequency_gemtek,
	mute_gemtek, volumeup_gemtek, mute_gemtek,
	state_gemtek,
	mono_stub,
	search_up_generic, search_down_generic},

	{ "rtII", radiotrack_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"AIMS Lab RadioTrack II FM Radio Card", 8750, 10800,
	Grab_Port_radiotrackII, Release_Port_radiotrackII,
	info_port_radiotrackII, get_freq_stub, get_vol_stub,
	Change_Frequency_radiotrackII,
	mute_radiotrackII, unmute_radiotrackII, mute_radiotrackII,
	state_radiotrackII,
	mono_radiotrackII,
	search_up_radiotrackII, search_down_radiotrackII},

	{ "sf2d", sf16fm_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"SoundForte Legacy 128 SF16-FMD2 Driver", 8750, 10800,
	Grab_Port_sf16fmd2, Release_Port_sf16fmd2, info_port_sf16fmd2,
	get_freq_stub, get_vol_stub,
	Change_Frequency_sf16fmd2,
	mute_sf16fmd2, volumeup_sf16fmd2, volumedn_sf16fmd2,
	state_sf16fmd2,
	mono_sf16fmd2,
	search_stub, search_stub},

        { "bmc", bmc_ports, INFO_ROOT | INFO_FREQ | 0x02,
        "BMC fcc-id HMA00-0076 Radio Card", 8750, 10800,
        Grab_Port_bmc, Release_Port_bmc, info_port_bmc,
        get_freq_bmc, get_vol_stub,
        Change_Frequency_bmc,
        mute_bmc, volumeup_bmc, volumedn_bmc,
        state_bmc,
        set_mono_bmc,
        search_stub, search_stub},

	{ "sfr", sf16fm_ports, INFO_ROOT | INFO_SCAN | INFO_FREQ | 0x02,
	"SoundForte RadioLink SF16-FMR FM Radio Card", 8750, 10800,
	Grab_Port_sf16fmr, Release_Port_sf16fmr, info_port_sf16fmr,
	get_freq_sf16fmr, get_vol_stub,
	Change_Frequency_sf16fmr,
	mute_sf16fmr, volumeup_sf16fmr, volumedn_sf16fmr,
	state_sf16fmr,
	mono_stub,
	search_stub, search_stub},

	{ "sf2r", pci_ports, INFO_ROOT | INFO_FREQ | 0x01,
	"SoundForte RadioLink SF16-FMR2 FM Radio Card", 8750, 10800,
	Grab_Port_sf16fmr2, Release_Port_sf16fmr2, info_port_sf16fmr2,
	get_frequency_sf16fmr2, get_vol_stub,
	set_frequency_sf16fmr2,
	mute_sf16fmr2, volumeup_sf16fmr2, volumedown_sf16fmr2,
	state_sf16fmr2, mono_sf16fmr2,
	search_up_sf16fmr2, search_down_sf16fmr2},

        { "tt", pci_ports, INFO_ROOT | 0x01,
        "Terratec Card", 8750, 10800,
        Grab_Port_tt, Release_Port_tt, info_port_tt,
        get_freq_stub, get_vol_stub,
        set_frequency_tt,
        mute_tt, volumeup_tt, volumedown_tt,
        state_tt, mono_tt,
        search_up_tt, search_down_tt},

	{ "sfi", sf16fm_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"SoundForte RadioX SF16-FMI Driver", 8750, 10800,
	Grab_Port_radiotrack, Release_Port_radiotrack, info_port_radiotrack,
	get_freq_stub, get_vol_stub,
	Change_Frequency_radiotrack,
	mute_radiotrack, volumeup_radiotrack, volumedn_radiotrack,
	state_radiotrack,
	mono_radiotrack,
	search_up_generic, search_down_generic},

	{ "mr", pci_ports, INFO_ROOT | 0x01,
	"Guillemot Maxi Radio FM2000 PCI Radio Card", 8750, 10800,
	Grab_Port_gtp, Release_Port_gtp, info_port_gtp,
	get_freq_stub, get_vol_stub, Change_Frequency_gtp,
	mute_gtp, unmute_gtp, mute_gtp, state_gtp, mono_gtp,
	search_up_gtp, search_down_gtp},

	{ "sf4r", pci_ports, INFO_ROOT | INFO_FREQ | 0x01,
	"SoundForte RadioLink SF64-PCR PCI FM Radio Card", 8750, 10800,
	Grab_Port_sf64pcr, Release_Port_sf64pcr, info_port_sf64pcr,
	get_frequency_sf64pcr, get_vol_stub,
	set_frequency_sf64pcr,
	mute_sf64pcr, unmute_sf64pcr, mute_sf64pcr,
	state_sf64pcr, mono_sf64pcr,
	search_up_generic, search_down_generic},

	{ "sqx", pci_ports, INFO_ROOT | INFO_FREQ | 0x01,
	"SoundForte Quad X-treme SF256-PCP-R Driver", 8750, 10800,
	Grab_Port_sf256pcpr, Release_Port_sf256pcpr, info_port_sf256pcpr,
	get_frequency_sf256pcpr, get_vol_stub,
	set_frequency_sf256pcpr,
	mute_sf256pcpr, unmute_sf256pcpr, mute_sf256pcpr,
	state_sf256pcpr, mono_sf256pcpr,
	search_up_sf256pcpr, search_down_sf256pcpr},

	{ "stx", pci_ports, INFO_ROOT | INFO_FREQ | 0x01,
	"SoundForte Theatre X-treme 5.1 SF256-PCS-R Driver", 8750, 10800,
	Grab_Port_sf256pcs, Release_Port_sf256pcs, info_port_sf256pcs,
	get_frequency_sf256pcs, get_vol_stub,
	set_frequency_sf256pcs,
	mute_sf256pcs, unmute_sf256pcs, mute_sf256pcs,
	state_sf256pcs, mono_sf256pcs,
	search_up_sf256pcs, search_down_sf256pcs},

#ifdef BSDBKTR
	{ "hx", bktr_ports, 0x1,
	"AIMS Lab Highway Xtreme TV card with FM tuner",
	8750, 10800, get_port_xtreme, free_port_xtreme, info_port_xtreme,
	get_freq_stub, get_vol_stub, set_frequency_xtreme,
	mute_xtreme, unmute_xtreme, mute_xtreme, state_xtreme,
	mono_xtreme, search_up_generic, search_down_generic},
#endif /* BSDBKTR */

#ifdef BSDRADIO
	{ "br", bktr_ports, INFO_VOLU | INFO_FREQ | 0x1,
	"OpenBSD and NetBSD FM Radio Card Driver", 8750, 10800,
	get_port_bsdradio, free_port_bsdradio, info_port_bsdradio,
	get_frequency_bsdradio, get_volume_bsdradio, set_frequency_bsdradio,
	mute_bsdradio, volumeup_bsdradio, volumedn_bsdradio,
	state_bsdradio, mono_bsdradio, search_up_generic, search_down_generic},
#endif /* BSDRADIO */

	{ "gtp", pci_ports, INFO_ROOT | 0x01, "Gemtek PCI FM Radio Card",
	8750, 10800, Grab_Port_gtp, Release_Port_gtp, info_port_gtp,
	get_freq_stub, get_vol_stub, Change_Frequency_gtp,
	mute_gtp, unmute_gtp, mute_gtp, state_gtp, mono_gtp,
	search_up_gtp, search_down_gtp}
};

#endif /* SPECS_H__ */
