/* Terratec Active Radio ISA
   for fmio
   (c) 09.06.2002 GM
   The user may choose between these 2 licenses at his will:
   Gnu Public License or BSD License
 */
/*

   Volume Control:
   uc is connected to 74hct164
   74hct164 is connected to two 4051bt (right and left audio)

   hef4051bt = 8-channel analogue multiplexer/demultiplexer
   74hct164  = 8-Bit Serial-In/Parallel-Out Shift Register

   Others:
   Octal D Flip-Flop w/Reset (74HCT273)
   74HC/HCT367 are hex non-inverting buffer/line drivers with 3-state outputs
   Quad 2-Input OR Gate (74HCT32).
   Quad Buffer; 3-State (74HCT125)

   RDS:
   SAA6588 

   Radio:
   OM5610 w tea5757

   ISAPNP-ID: TER2111

   Sources: "radio-terratec.c" from linux-2.4.xx, fmio sources

   Todo:
    Fix fmio volume handling.
	- Max Vol. when nothing is given on the commandline
		(so a user hasn't to figure out he needs to set volume first)
*/

#include "access.h"
#include "tea5757.h"

/* Tea5757 bus */
int TEA_data = 4;
int TEA_clk  = 8;
int TEA_wren = 0x10;

static struct tea5757_t card;
static unsigned long radioport = 0;
static int volume = 0;

static void write_shift_register(unsigned long);
static unsigned long read_shift_register(void);
void set_volume_tt(int);


/******************************************************************/

int
Grab_Port_tt(unsigned long port) {
	card.frequency = 0;
	card.search = TEA5757_SEARCH_END;
	card.sensitivity = TEA5757_S030;
	card.stereo = TEA5757_STEREO;
	card.write = write_shift_register;
	card.read = read_shift_register;
	radioport = 0x590;

	if(radio_get_iopl() < 0)
		return -1;

	OUTB(radioport,0);
	if ((inb(radioport) & 0x0f) != 0x08) {
		/* only bits 0-2 are writeable here; bit 3 is always 1 */
		warnx("Terratec ActiveRadio ISA must be activated by isapnp!");
		return -1;
	}

	return 0;
}

int
Release_Port_tt(void) {
	return radio_release_iopl();
}

unsigned long
info_port_tt(void) {
	return radioport;
}

void
set_frequency_tt(unsigned long frequency) {
	card.frequency = frequency;
	card.search = TEA5757_SEARCH_END;
	tea5757_write_shift_register(&card);

	set_volume_tt(7);

	return;
}

unsigned long
search_up_tt(unsigned long freq) {
	card.frequency = freq;
	card.search = TEA5757_SEARCH_UP;
	return tea5757_search(&card);
}

unsigned long
search_down_tt(unsigned long freq) {
	card.frequency = freq;
	card.search = TEA5757_SEARCH_DOWN;
	return tea5757_search(&card);
}

void
mute_tt(void) {
	volume = 0;
	set_volume_tt(volume);
}

void
volumeup_tt(void) {
	if(volume < 7)
		volume++;
	set_volume_tt(volume);
}

void
volumedown_tt(void) {
	if(volume > 0)
		volume--;
	set_volume_tt(volume);
}

void set_volume_tt(int volume) {
        int i;

        volume = volume + (volume * 32); /* change both channels */
        for (i = 0; i < 8; i++) {
                if (volume & (0x80 >> i))
                        OUTB(0x80, radioport + 1);
                else OUTB(0x00, radioport + 1);
        }
}


int
state_tt(void) {
	return 3;
}

void
mono_tt(void) {
	card.stereo = TEA5757_MONO;
}


/*
 * it is not possible to read back from tea5757
 * with my terratec activeradio isa
 */
static unsigned long
read_shift_register(void) {
	return 0;
}

static void
write_shift_register(unsigned long data) {
    int c = 25;

	OUTB(radioport, TEA_wren);

	while (c--)
		if (data & (1 << c)) {
			OUTB(radioport, TEA_wren | TEA_data);
			OUTB(radioport, TEA_wren | TEA_clk | TEA_data);
			OUTB(radioport, TEA_wren | TEA_data);
		} else {
			OUTB(radioport, TEA_wren);
			OUTB(radioport, TEA_wren | TEA_clk);
			OUTB(radioport, TEA_wren);
		}
	OUTB(radioport, 0);
}
