/*
 * Copyright (c) 2001, 2002 Vladimir Popov <jumbo@narod.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * bktrctl - holds /dev/tuner open all the time
 * $Id: bktrctl.c,v 1.1 2002/01/04 09:28:11 pva Exp $
 */

#include <sys/param.h>

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/stat.h>

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>

#include "../access.h"

#ifdef linux
#define	TUNER_DEVICE_1	"/dev/radio"
#define TUNER_DEVICE_2	"/dev/radio0"
#else
#define TUNER_DEVICE_1	"/dev/tuner"
#define TUNER_DEVICE_2	"/dev/tuner0"
#endif

#define SYMLINK_DEPTH	10

static int fd = -1;

int
mute(void) {
#ifdef linux
	struct video_audio va;

	va.audio = 0;
	va.flags = VIDEO_AUDIO_MUTE;
	va.volume = 0;

	if (ioctl(fd, VIDIOCSAUDIO, &va) < 0)
#else
	int intern = AUDIO_MUTE;
	if (ioctl(fd, BT848_SAUDIO, &intern) < 0)
#endif /* linux */
		return -1;

	return 0;
}

int
open_device(void) {
	char buf[PATH_MAX], next_buf[PATH_MAX];
	struct stat s;
	int rfnb = 0; /* Length of file name read from readlink() */
	int depth = 0, found = 0;

	strncpy(buf, TUNER_DEVICE_1, PATH_MAX - 1);
	buf[PATH_MAX - 1] = '\0';

	/* Follow symlinks until the file is found */
	for (depth = 0; depth < SYMLINK_DEPTH; depth++) {

		if (lstat(buf, &s) < 0)
			break;

		if (!S_ISLNK(s.st_mode)) {
			found = 1;
			break;
		}

		rfnb = readlink(buf, next_buf, PATH_MAX - 1);
		if (rfnb < 0 || rfnb == 0)
			break;
		next_buf[rfnb] = '\0';

		strncpy(buf, next_buf, PATH_MAX - 1);
		buf[PATH_MAX - 1] = '\0';
	}

	/*
	 * Ok, TUNER_DEVICE_1 doesn't exist or is buried too deep
	 * Let's try last resort -- TUNER_DEVICE_2
	 */
	if (!found) {
		strncpy(buf, TUNER_DEVICE_2, PATH_MAX - 1);
		buf[PATH_MAX - 1] = '\0';
	}

#if 0
	if ((fd = open(buf, O_RDONLY)) < 0)
#else
	if ((fd = open(buf, O_RDWR)) < 0)
#endif
		return -1;

	return mute();
}

int
main() {
	int rfd;
	struct rlimit rlim;

	if (getppid() != 1) {
		signal(SIGTTOU, SIG_IGN);
		signal(SIGTTIN, SIG_IGN);
		signal(SIGTSTP, SIG_IGN);

		if (fork() != 0)
			exit(errno);

		setsid();
	}

	getrlimit(RLIMIT_NOFILE, &rlim);
	for (rfd = 0; rfd < rlim.rlim_max; rfd++)
		close(rfd);

	chdir("/");

	if (open_device() < 0)
		exit(errno);

	while (1)
		usleep(900000);

	return 0;
}
