/*
 * Copyright (c) 2000, 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: zoltrix.c,v 1.23 2002/01/14 10:21:41 pva Exp $
 * ALPHA
 * zoltrix driver for fmio
 */

#include "access.h"

#define WRITE(a)	usleep(0); OUTB(radioport, a)

static int vol = 0;
static int stereo = 0; /* Use stereo by default */
static unsigned long radioport = 0;

/******************************************************************/

int
Grab_Port_zoltrix(unsigned long port) {
	radioport = port;
	return radio_get_ioperms(radioport, 0x0F);
}

int
Release_Port_zoltrix(void) {
	return radio_release_ioperms(radioport, 0x0F);
}

unsigned long
info_port_zoltrix(void) {
	return radioport;
}

void
Change_Frequency_zoltrix(unsigned long frequency) {
	/* tunes the radio to the desired frequency */
	unsigned long long bitmask, f;
	int i;
	float freq = frequency/100;

	f = (unsigned long long)(((float)(freq-88.0))*200.0)+0x4d1c;
	i = 45;
	bitmask = 0xc480402c10080000ull;
	bitmask = (bitmask^((f&0xff)<<47)^((f&0xff00)<<30)^(stereo<<31));

	WRITE(0x0);
	WRITE(0x0);
	inb(radioport+3);

	WRITE(0x40);
	WRITE(0xc0);
	while (i--) {
		if ((bitmask & 0x8000000000000000ull) != 0) {
			WRITE(0x80);
			WRITE(0x00);
			WRITE(0x80);
		} else {
			WRITE(0xc0);
			WRITE(0x40);
			WRITE(0xc0);
		}
		bitmask *= 2;
	}

	/* Termination sequence */
	WRITE(0x80);
	WRITE(0xc0);
	WRITE(0x40);
	usleep(20000);
	if (vol) { WRITE(vol); }
	usleep(10000);
	inb(radioport+2);

	return;
}

void
mute_zoltrix(void) {
	OUTB(radioport, 0);
	usleep(0);
	OUTB(radioport, 0);
	inb(radioport+3);
}

void
volumeup_zoltrix(void) {
	if (vol < 16)
		vol++;

	OUTB(radioport, vol);
	usleep(10000);
	inb(radioport+2);
}

void
volumedn_zoltrix(void) {
	if (vol > 0)
		vol--;
	
	OUTB(radioport, vol);
	usleep(10000);
	inb(radioport+2);
}

int
state_zoltrix(void) {
	int a, b;

	OUTB(radioport, 0);
	OUTB(radioport, vol);
	usleep(10000);

	a = inb(radioport);
	usleep(1000);
	b = inb(radioport);
	
	if (a == b) {
		switch (a) {
			case 0xcf: return 0;
			case 0xdf: return 1;
			case 0xef: return 2;
			default: return 3;
		}
	}

	return 3;
}

void
mono_zoltrix(void) {
	stereo = 1;
}
